import Ember from 'ember';

const {
  meta
} = Ember;

let id = 0;
const dateKey = new Date().getTime();
const metaKey = symbol();

function UNDEFINED() {}

function symbol() {
  return `__ember${dateKey}${id++}`;
}

class WeakMap {
  constructor(iterable = []) {
    this._id = symbol();

    for (let iteration of iterable) {
      const [key, value] = iteration;

      if (typeof iteration !== 'object') {
        throw new TypeError(`Iterator value ${key} is not an entry object`);
      }

      this.set(key, value);
    }
  }

  /*
   * @method get
   * @param key {Object}
   * @return {*} stored value
   */
  get(obj) {
    var metaInfo   = meta(obj);
    var metaObject = metaInfo[metaKey];

    if (metaInfo && metaObject) {
      if (metaObject[this._id] === UNDEFINED) {
        return undefined;
      }

      return metaObject[this._id];
    }
  }

  /*
   * @method set
   * @param key {Object}
   * @param value {Any}
   * @return {Any} stored value
   */
  set(obj, value) {
    var type = typeof obj;
    if (!obj || (type !== 'object' && type !== 'function')) {
      throw new TypeError('Invalid value used as weak map key');
    }

    var metaInfo = meta(obj);
    if (value === undefined) {
      value = UNDEFINED;
    }

    if (!metaInfo[metaKey]) {
      metaInfo[metaKey] = {};
    }

    metaInfo[metaKey][this._id] = value;
    return this;
  }

  /*
   * @method has
   * @param key {Object}
   * @return {Boolean} if the key exists
   */
  has(obj) {
    var metaInfo   = meta(obj);
    var metaObject = metaInfo[metaKey];

    return (metaObject && metaObject[this._id] !== undefined);
  }

  /*
   * @method delete
   * @param key {Object}
   */
  delete(obj) {
    var metaInfo = meta(obj);

    if (this.has(obj)) {
      delete metaInfo[metaKey][this._id];

      return true;
    }

    return false;
  }
}

export default (Ember.WeakMap) ? Ember.WeakMap : WeakMap;
